package visualization6;

/**
 * はてなブックマークエントリーに対応する末端ノードです。
 */

public class BookmarkItem extends Item {
    private Bookmark bookmark;
    private BookmarkDetail detail;

    public BookmarkItem(Bookmark bookmark, BookmarkDetail detail,
            IndexMapper mapper) {
        // ブックマーク数をそのまま面積化すると比率が極端になるので
        // 平方根をとって調整する
        super(bookmark.title, tagsToVector(detail, mapper), Math
                .sqrt(detail.bookmarkCount));
        this.bookmark = bookmark;
        this.detail = detail;
    }

    private static MultiVector tagsToVector(BookmarkDetail detail,
            IndexMapper mapper) {
        MultiVector vector = new MultiVector(mapper.size());
        for (String tag : detail.tags) {
            if (mapper.contains(tag)) {
                int index = mapper.get(tag);
                vector.set(index, vector.get(index) + 1);
            }
        }
        vector.normalize(); // ベクトルを正規化する
        return vector;
    }

    public Bookmark getBookmark() {
        return bookmark;
    }

    public BookmarkDetail getDetail() {
        return detail;
    }
}
