package visualization6;

import java.util.ArrayList;
import java.util.List;

/**
 * クラスタに対応するノードです。
 */

public class Cluster implements Node {
    private Node left;
    private Node right;
    private List<MultiVector> cachedVectors;

    /**
     * ノードを作成します。
     * 
     * @param left
     *            子ノード(左)
     * @param right
     *            子ノード(右)
     */
    public Cluster(Node left, Node right) {
        this.left = left;
        this.right = right;
    }

    /**
     * 子ノード(左)を取得します。
     * 
     * @return ノード
     */
    public Node getLeft() {
        return left;
    }

    /**
     * 子ノード(右)を取得します。
     * 
     * @return ノード
     */
    public Node getRight() {
        return right;
    }

    public List<MultiVector> getVectors() {
        // 高速化のため結果をキャッシュする
        if (cachedVectors == null) {
            cachedVectors = new ArrayList<MultiVector>();
            // leftノードとrightノードのベクトル集合を連結
            cachedVectors.addAll(left.getVectors());
            cachedVectors.addAll(right.getVectors());
        }
        return cachedVectors;
    }

    public double getArea() {
        return left.getArea() + right.getArea();
    }
}
